# -*- coding: utf-8 -*-
# Copyright (C) 2014-2024  EDF
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See https://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
# Author : Konstantin Leontev (OpenCascade S.A.S)

"""Example of algorithm script for GEOM Merge Faces plugin.
"""

import sys
import logging
from time import sleep

import salome

from salome.geom import geomBuilder
from qtsalome import QFileDialog, QApplication, pyqtSignal


salome.salome_init()
geompy = geomBuilder.New()


def run(args_dict, progress_emitter):
    """
    Helper function to call run() with arguments parsed from dictionary.

    Args:
        args_dict - arguments as pairs string : any type value

    Returns:
        A result object.
    """

    logging.info('Run Merge Faces algorithm.')
    progress_emitter.emit()


    if ('source_solid' not in args_dict or
        'selected_ids' not in args_dict or
        'result_name' not in args_dict or
        'precision' not in args_dict):

        logging.info('Cant execute an algo because the arguments are empty!')
        return False

    source_solid = args_dict['source_solid']
    faces_ids = args_dict['selected_ids']
    result_name = args_dict['result_name']


    logging.info('Creating of two faces...')
    progress_emitter.emit()

    # This block creates a face using passed selected faces.
    # Commented to simplify output - just one object.

    # # Fusion des deux faces
    # faces = geompy.SubShapes(source_solid, faces_ids)
    # logging.info('faces: %s', faces)
    # partition = geompy.MakePartition([faces[0], faces[1]],[])
    # points = [geompy.GetVertexNearPoint(partition, geompy.MakeVertex(-298, 29, 250)),
    #         geompy.GetVertexNearPoint(partition, geompy.MakeVertex(178, 29, 250)),
    #         geompy.GetVertexNearPoint(partition, geompy.MakeVertex(178, -282, 250)),
    #         geompy.GetVertexNearPoint(partition, geompy.MakeVertex(-298, -282, 250))]
    # wire = geompy.MakePolyline(points,True)
    # fused_face = geompy.MakeFaceWires([wire], True)
    # geompy.addToStudy(fused_face, "fused_face")

    logging.info('Creating of a new geometry from the source brep...')
    progress_emitter.emit()

    sleep(1)

    # Fusion des deux faces au sein de la boite + nettoyage de la boite
    points = [geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(-298, 29, 250)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(178, 29, 250)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(178, -282, 250)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(-298, -282, 250)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(-298, 29, 0)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(178, 29, 0)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(178, -282, 0)),
            geompy.GetVertexNearPoint(source_solid, geompy.MakeVertex(-298, -282, 0))]
    # ### Fusion des deux faces
    wire = geompy.MakePolyline(points[:4],True)
    faces = [geompy.MakeFaceWires([wire], True)]

    logging.info('Cleaning of the new geometry...')
    progress_emitter.emit()

    sleep(1)

    # Uncomment to simulate exception handling in a thread worker class
    # raise Exception

    # ### Nettoyage des 4 faces latérales
    wire = geompy.MakePolyline([points[3], points[2], points[6], points[7]],True)
    faces.append(geompy.MakeFaceWires([wire], True))
    wire = geompy.MakePolyline([points[0], points[3], points[7], points[4]],True)
    faces.append(geompy.MakeFaceWires([wire], True))
    wire = geompy.MakePolyline([points[1], points[0], points[4], points[5]],True)
    faces.append(geompy.MakeFaceWires([wire], True))
    wire = geompy.MakePolyline([points[2], points[1], points[5], points[6]],True)
    faces.append(geompy.MakeFaceWires([wire], True))

    # ### Récupération de la dernière face
    faces.append(geompy.GetFaceNearPoint(source_solid, geompy.MakeVertex(-59, -127, 0)))

    logging.info('Creating a solid...')
    progress_emitter.emit()

    sleep(1)

    # ### Création du solide
    shell = geompy.MakeShell(faces)
    solid = geompy.MakeSolid(shell)

    geompy.addToStudy(solid, result_name)

    logging.info('Merge Faces algorithm was completed successfully.')
    progress_emitter.emit()

    return solid


def test():
    """
    Tests execution of repair algo script.
    """

    logging.basicConfig(level=logging.DEBUG)

    cube_file, _ = QFileDialog.getOpenFileName(None, 'Open brep', '/home', 'Brep Files (*.brep)')
    if not cube_file:
        return

    # cube_file = "PartitionCube.brep"
    source_solid = geompy.ImportBREP(cube_file)

    # Récupération des faces à fusionner
    face_a = geompy.GetFaceNearPoint(source_solid, geompy.MakeVertex(-143, -127, 250))
    face_b = geompy.GetFaceNearPoint(source_solid, geompy.MakeVertex(49,-127,250))

    geompy.addToStudy(source_solid, "source_solid")
    geompy.addToStudyInFather(source_solid, face_a, "face_a")
    geompy.addToStudyInFather(source_solid, face_b, "face_b")


    args_dict = {
            'source_solid': source_solid,
            'selected_ids': [face_a, face_b],
            'result_name': 'MergeFaces_result',
            'precision': 0.1
    }

    # Dummy emitter
    # TODO: doesn't work
    # progress_emitter = pyqtSignal()
    progress_emitter = type('DummyEmitter', (object,), {'emit': lambda self: sleep(0.1)})()

    run(args_dict, progress_emitter)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    test()
    sys.exit(app.exec_())
